@@ -2,8 +2,7 @@ logLik.rma <- function(object, REML, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(object, "rma"))
-      stop(mstyle$stop("Argument 'object' must be an object of class \"rma\"."))
+   .chkclass(class(object), must="rma")
 
    if (missing(REML)) {
       if (object$method == "REML") {
