@@ -2,8 +2,7 @@ print.list.confint.rma <- function(x, digits=x$digits, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(x, "list.confint.rma"))
-      stop(mstyle$stop("Argument 'x' must be an object of class \"list.confint.rma\"."))
+   .chkclass(class(x), must="list.confint.rma")
 
    digits <- .get.digits(digits=digits, xdigits=x$digits, dmiss=FALSE)
 
